import numpy as np
import scipy as sc
import glob2
import matplotlib.pyplot as plt
import seaborn as sb
import natsort
import pandas as pd

V = 2592
H = 1944
angle = 30 * np.pi / 180      # Angle of incidence in degrees
alpha = 16.647
cropH = 800
cropVr = 0
cropVl = 0
y_res = 0.200
x_res = 0.500
nb_sample = (V - cropVr - cropVl) // 5
nb_frames = 54
x_length = nb_frames * x_res
y_length = nb_sample * y_res
print("X: ", x_length)
print("Y: ", y_length)

buffer = np.zeros((V, nb_frames), dtype=float)

lineList4 = np.zeros((V - cropVr - cropVl, nb_frames), dtype=float)   # List made of each frame's laser line
heightMap = np.zeros((nb_sample, nb_frames), dtype=float)

files4 = glob2.glob(r'C:\Users\Max\Documents\Ulg\TFE\TFE\Mesures\2D\still\recap\elcoM1\par\Fline*.txt')

files4 = natsort.natsorted(files4, key=lambda y: y.lower())
for count, fname in enumerate(files4):
    buffer[:, count] = np.loadtxt(fname)

lineList4 = buffer[cropVl:V-cropVr]
print(len(lineList4))
sampled = sc.signal.resample(lineList4, nb_sample)
print(len(sampled))

x = np.arange(0, x_length, x_res)
y = np.arange(0, y_length, y_res)
X, Y = np.meshgrid(x, y)
x_shift, y_shift = np.meshgrid(x, y)

floor = np.max(sampled) # Highest line position value, corresponding to the lowest height mesured or to unwanted reflections
avg = np.mean(sampled)
ceil = np.min(sampled)


for j in range(0, nb_frames):
    for i in range(0, nb_sample):
        heightMap[i, j] = (floor - sampled[i, j]) * alpha + 50
#     # plt.plot(y[700:], test[700:, j])
#     # plt.show()

ax = plt.axes(projection='3d')
ax.plot_surface(X, Y, heightMap, rstride=1, cstride=1,
                cmap='viridis', edgecolor='none')
#ax.set_title('surface')
ax.view_init(0, 90)
ax.set_xlabel('x(mm)')
ax.set_ylabel('y(mm)')
ax.set_zlabel('z(µm)')
plt.show()

df_cm = pd.DataFrame(heightMap, index = y,
                  columns = x)
ax = sb.heatmap(df_cm, annot=False, xticklabels=5, yticklabels=20, cbar_kws={'label': 'z (µm)'})
plt.xlabel('x (mm)')
plt.ylabel('y (mm)')
plt.show()

import matplotlib.pyplot as plt
import seaborn as sb
import natsort
import pandas as pd

V = 2592
H = 1944
angle = 30 * np.pi / 180      # Angle of incidence in degrees
alpha = 16.647
cropH = 800
cropVr = 0
cropVl = 0
y_res = 0.200
x_res = 0.500
nb_sample = (V - cropVr - cropVl) // 5
nb_frames = 54
x_length = nb_frames * x_res
y_length = nb_sample * y_res
print("X: ", x_length)
print("Y: ", y_length)

buffer = np.zeros((V, nb_frames), dtype=float)

lineList4 = np.zeros((V - cropVr - cropVl, nb_frames), dtype=float)   # List made of each frame's laser line
heightMap = np.zeros((nb_sample, nb_frames), dtype=float)

files4 = glob2.glob(r'C:\Users\Max\Documents\Ulg\TFE\TFE\Mesures\2D\still\recap\elcoM1\par\Fline*.txt')

files4 = natsort.natsorted(files4, key=lambda y: y.lower())
for count, fname in enumerate(files4):
    buffer[:, count] = np.loadtxt(fname)

lineList4 = buffer[cropVl:V-cropVr]
print(len(lineList4))
sampled = sc.signal.resample(lineList4, nb_sample)
print(len(sampled))

x = np.arange(0, x_length, x_res)
y = np.arange(0, y_length, y_res)
X, Y = np.meshgrid(x, y)
x_shift, y_shift = np.meshgrid(x, y)

floor = np.max(sampled) # Highest line position value, corresponding to the lowest height mesured or to unwanted reflections
avg = np.mean(sampled)
ceil = np.min(sampled)


for j in range(0, nb_frames):
    for i in range(0, nb_sample):
        heightMap[i, j] = (floor - sampled[i, j]) * alpha + 50
#     # plt.plot(y[700:], test[700:, j])
#     # plt.show()

ax = plt.axes(projection='3d')
ax.plot_surface(X, Y, heightMap, rstride=1, cstride=1,
                cmap='viridis', edgecolor='none')
#ax.set_title('surface')
ax.view_init(0, 90)
ax.set_xlabel('x(mm)')
ax.set_ylabel('y(mm)')
ax.set_zlabel('z(µm)')
plt.show()

df_cm = pd.DataFrame(heightMap, index = y,
                  columns = x)
ax = sb.heatmap(df_cm, annot=False, xticklabels=5, yticklabels=20, cbar_kws={'label': 'z (µm)'})
plt.